import re
import os
import sys
sys.path.append(r'c:\\Python27\\Lib\\')
sys.path.append(r'c:\\Python27\\Lib\\site-packages\\')
sys.path.append(r'c:\\Python27\\Lib\\site-packages\\jira')
sys.path.append(r'c:\\Python27\\Lib\\site-packages\\requests')
sys.path.append(r'c:\\Python27\\Lib\\site-packages\\simplejson')
from jira.client import JIRA

BAD_COMMIT = True
OK = False

class JiraCommitHook:
    """ 
    Class JiraCommitHook checks if a valid Jira issue is provided
    """
    def __init__(self, ui, repo):
        self.jiraKeys=[]
        self.repo=repo
        self.ui=ui
        self.jiraIssue=''
        mJirakeyFile=os.path.join(repo.root,'.jirakey')
        with open (mJirakeyFile, "r") as myfile:
            self.jiraKeys=myfile.read().replace('\n', '').split(',')
            print self.jiraKeys
        mJiraOptions = {
            'server': self.ui.config('jira', 'jira.url',
                             default='http://myjira.dmn.local',untrusted=False)
        }
        self.jira = JIRA(options=mJiraOptions,basic_auth=('user','passwd'))

    def checkJira(self):
        print self.jiraIssue
        print self.ui.username()
        projects = self.jira.projects()
        mIssue = self.jira.issue(self.jiraIssue)
        print mIssue
        return False

    def checkMessage(self, msg):
        """
        Checks the actual commit message.
        """
        mIsCorrect = False
        mReNames = '|'.join(['(%s-\d+)' % name for name in self.jiraKeys])
        mP = re.compile(r"^%s" % mReNames)
        mRes = mP.search(msg)
        if mRes:
            self.jiraIssue = mRes.group(0)
            if (self.checkJira() == True):
                mIsCorrect = True
        return mIsCorrect

    def printUsage(self):
        self.ui.warn('=====\n')
        self.ui.warn('Commit message must have a valid JIRA issue key\n')
        self.ui.warn('Example:\n')
        self.ui.warn('PRJ-42 - the answer to life, universe and everything \n')
        self.ui.warn('=====\n')

def checkCommitMessage(ui, repo, **kwargs):
    """
    Checks a single commit message for adherence to commit message rules.

    To use add the following to your project .hg/hgrc for each
    project you want to check, or to your user hgrc to apply to all projects.

    [hooks]
    pretxncommit = python:path/to/script/enforce-message.py:checkCommitMessage
    """
    mCommitHook = JiraCommitHook(ui, repo)
    hg_commit_message = repo['tip'].description()
    if(mCommitHook.checkMessage(hg_commit_message) == False):
        mCommitHook.printUsage()
        return BAD_COMMIT
    else:
        return OK

def checkAllCommitMessage(ui, repo, node, **kwargs):
    """
    Checks all inbound changeset messages from a push for adherence to the commit message rules.

    [hooks]
    pretxnchangegroup = python:path/to/script/enforce-message.py:checkAllCommitMessage
    """
    mCommitHook = JiraCommitHook(ui, repo)
    for rev in xrange(repo[node].rev(), len(repo)):
        message = repo[rev].description()
        if(mCommitHook.checkMessage(message) == False):
            ui.warn("Revision "+str(rev)+" commit message:["+message+"] does not adhere to commit message rules\n")
            mCommitHook.printUsage()
            return BAD_COMMIT
    return OK

