#include <jansson.h>

#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

#include "auth.h"
#include "lastfm.h"

LASTFM_AUTH *lastfm_auth_new(char *api_key, char *secret)
{
    LASTFM_AUTH *auth;
    
    auth = malloc(sizeof(LASTFM_AUTH));
    auth->api_key = api_key;
    auth->secret = secret;
    auth->session = NULL;
}

void lastfm_auth_delete(LASTFM_AUTH *auth)
{
    if(auth == NULL)
    {
        return;
    }

    if(auth->session != NULL)
    {
        free(auth->session);
    }
    
    free(auth);
}

char *lastfm_auth_getToken(const LASTFM_AUTH *auth)
{
    char *response, *json_token, *token;
    json_t *json_root;
    json_error_t json_error;
    
    if(auth == NULL)
    {
        return NULL;
    }
    
    response = lastfm_method("auth.getToken", auth);
    json_root = json_loads(response, 0, &json_error);
    json_unpack_ex(json_root, &json_error, JSON_STRICT, "{s:s}",
                   "token", &json_token);
                   
    asprintf(&token, "%s", json_token);
                   
    json_decref(json_root);
                   
    return token;
}

LASTFM_SESSION *lastfm_auth_getSession(const LASTFM_AUTH *auth)
{
    LASTFM_SESSION *session;
    char *post, *response;
    char *session_name, *session_key, *session_subscriber;
    json_t *json_root;
    json_error_t json_error;
    
    if(auth == NULL)
    {
        return NULL;
    }
    
    asprintf(&post, "auth.getSession&token=%s", auth->token);
    response = lastfm_method(post, auth);
    
    json_root = json_loads(response, 0, &json_error);
    if(json_root == NULL)
    {
        return NULL;
    }
    
    if(json_object_get(json_root, "error") != NULL)
    {
        return NULL;
    }
    
    json_unpack_ex(json_root, &json_error, JSON_STRICT, "{s:{s:s, s:s, s:s}}",
                   "session",
                   "name", &session_name,
                   "key", &session_key,
                   "subscriber", &session_subscriber);
    
    session = malloc(sizeof(LASTFM_SESSION));
    asprintf(&(session->name), "%s", session_name);
    asprintf(&(session->key), "%s", session_key);
    session->subscriber = atoi(session_subscriber);
    
    json_decref(json_root);
                   
    return session;
}

char *lastfm_auth_getUrl(const LASTFM_AUTH *auth)
{
    char *auth_url;
    
    if(auth == NULL)
    {
        return NULL;
    }
    
    asprintf(&auth_url, "http://www.last.fm/api/auth/?api_key=%s&token=%s", auth->api_key, auth->token);
    
    return auth_url;
}
