#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

#include <jansson.h>

#include "helper/list.h"

#include "lastfm.h"
#include "auth.h"
#include "lastfm.h"
#include "buylink_private.h"

#include "helper/list.h"

LASTFM_PRICE *price_new(void)
{
    LASTFM_PRICE *price;
    
    price = malloc(sizeof(LASTFM_PRICE));
    
    price->currency = NULL;
    price->formatted = NULL;
    
    return price;
}

void price_delete(LASTFM_PRICE *price)
{
    if(price == NULL)
    {
        return;
    }
    
    free(price->currency);
    free(price->formatted);
    free(price);
    
    return;
}

LASTFM_BUYLINK *buylink_new(void)
{
    LASTFM_BUYLINK *buylink;
    
    buylink = malloc(sizeof(LASTFM_BUYLINK));
    
    buylink->supplierName = NULL;
    buylink->buyLink = NULL;
    buylink->supplierIcon = NULL;
    
    buylink->price = price_new();
    
    return buylink;
}

void buylink_delete(LASTFM_BUYLINK *buylink)
{
    if(buylink == NULL)
    {
        return;
    }
    
    free(buylink->supplierName);
    free(buylink->buyLink);
    free(buylink->supplierIcon);
    price_delete(buylink->price);
    free(buylink);
    
    return;
}

LASTFM_BUYLINKS *buylinks_new(void)
{
    LASTFM_BUYLINKS *buylinks;
    
    buylinks = malloc(sizeof(LASTFM_BUYLINKS));
    buylinks->physicals = list_new();
    buylinks->downloads = list_new();
    buylinks->physicals->free_handler = buylinks->downloads->free_handler = (list_free_handler_t)buylink_delete;
    
    return buylinks;
}

void lastfm_buylinks_delete(LASTFM_BUYLINKS *buylinks)
{
    if(buylinks == NULL)
    {
        return;
    }
    
    list_delete(buylinks->physicals);
    list_delete(buylinks->downloads);
    
    return;
}

void getBuyLinks_parse(LIST *buylinks, json_t *json_buylinks, LASTFM_BUYLINK_TYPE buylink_type)
{
    size_t json_buylinks_size, i;
    json_error_t json_error;
    
    json_buylinks_size = json_array_size(json_buylinks);

    for(i = 0; i < json_buylinks_size; ++i)
    {
        LASTFM_BUYLINK *buylink;
        json_t *json_buylink;
        char *buylink_supplierName, *buylink_price_currency, *buylink_amount, *buylink_isSearch,
             *buylink_price_formatted, *buylink_buyLink, *buylink_supplierIcon;
        
        buylink = buylink_new();
        buylink->type = buylink_type;
        
        json_buylink = json_array_get(json_buylinks, i);
        json_unpack_ex(json_buylink, &json_error, JSON_STRICT, "{s:s, s?{s:s, s:s, s:s}, s:s, s:s, s:s}",
                       "supplierName", &buylink_supplierName,
                       "price",
                       "currency", &buylink_price_currency,
                       "amount", &buylink_amount,
                       "formatted", &buylink_price_formatted,
                       "buyLink", &buylink_buyLink,
                       "supplierIcon", &buylink_supplierIcon,
                       "isSearch", &buylink_isSearch);
                    
        asprintf(&(buylink->supplierName), "%s", buylink_supplierName);
        asprintf(&(buylink->buyLink), "%s", buylink_buyLink);
        asprintf(&(buylink->supplierIcon), "%s", buylink_supplierIcon);
        buylink->isSearch = atoi(buylink_isSearch);
        if(buylink->price->formatted != NULL)
        {
            asprintf(&(buylink->price->currency), "%s", buylink_price_currency);
            buylink->price->amount = atof(buylink_amount);
            asprintf(&(buylink->price->formatted), "%s", buylink_price_formatted);
        }
        else
        {
            price_delete(buylink->price);
            buylink->price = NULL;
        }
        
        list_add(buylinks, buylink);
    }
    
    return;
}

LASTFM_BUYLINKS *getBuyLinks(const char *artist,
                             const char *album,
                             const char *track,
                             const char *mbid,
                             const char *country,
                             const bool autocorrect,
                             const LASTFM_AUTH *auth,
                             const PAGE_TYPE page_type)
{
    char *method, *response;
    const char *package, *key[2], *value[2];
    LASTFM_BUYLINKS *buylinks;
    json_t *json_root, *json_buylinks;
    json_error_t json_error;
    
    memset(key, 0, sizeof(key));
    memset(value, 0, sizeof(value));
    
    switch(page_type)
    {
        case PAGE_ALBUM:
        {
            package = "album";
        }
        break;
        case PAGE_TRACK:
        {
            package = "track";
        }
        break;
    }
    
    if(mbid == NULL)
    {
        key[0] = "&artist=";
        value[0] = artist;
    
        switch(page_type)
        {
            case PAGE_ALBUM:
            {
                key[1] = "&album=";
                value[1] = album;
            }
            break;
            case PAGE_TRACK:
            {
                key[1] = "&track";
                value[1] = track;
            }
            break;
        }
    }
    else
    {
        key[0] = "&mbid=";
        value[0] = mbid;
    }
    
    asprintf(&method, "%s.getBuyLinks%s%s%s%s&autocorrect=%d&country=%s",
             package, key[0], value[0], key[1], value[1], autocorrect, country);
    response = lastfm_method(method, auth);
    
    json_root = json_loads(response, 0, &json_error);
    json_root = json_object_get(json_root, "affiliations");

    buylinks = buylinks_new();
    getBuyLinks_parse(buylinks->physicals, json_object_get(json_object_get(json_root, "physicals"), "affiliation"), LASTFM_BUYLINK_PHYSICAL);
    getBuyLinks_parse(buylinks->downloads, json_object_get(json_object_get(json_root, "downloads"), "affiliation"), LASTFM_BUYLINK_DOWNLOAD);
    
    json_decref(json_root);
    free(method);
    free(response);
    
    return buylinks;
}
