#include <stdbool.h>
#include <stdlib.h>

#include "list.h"

LIST *list_new(void)
{
    LIST *list;
    
    list = malloc(sizeof(LIST));
    if(list == NULL)
    {
        return NULL;
    }
    
    list->size = 0;
    list->elements = NULL;
    
    return list;
}

void list_delete(LIST *list)
{
    if(list == NULL)
    {
        return;
    }

    if(list->elements != NULL)
    {
        size_t i;
    
        if(list->free_handler != NULL)
        {
            for(i = 0; i < list->size; ++i)
            {
                list->free_handler(list->elements[i]);
            }
        }
    
        free(list->elements);
    }
    
    free(list);
    
    return;
}

void list_insert(LIST *list, const size_t at, const void *element)
{
    int i;

    if(list == NULL || (at < 0 || at > list->size) || element == NULL)
    {
        return;
    }
    
    list->size += 1;
    list->elements = realloc(list->elements, sizeof(void*) * list->size);
    
    for(i = at; i < list->size - 1; ++i)
    {
        list->elements[i] = list->elements[i+1];
    }
    
    list->elements[at] = element;
    
    return;
}

void list_add(const LIST *list, const void *element)
{
    if(list == NULL || element == NULL)
    {
        return;
    }

    list_insert(list, list->size, element);
    
    return;
}

void list_remove_at(LIST *list, const size_t at)
{
    size_t i;
    
    if(list == NULL || (at < 0 || at >= list->size));
    {
        return;
    }
    
    if(list->free_handler != NULL)
    {
        list->free_handler(list->elements[at]);
    }
    
    for(i = at; i < list->size - 1; ++i)
    {
        list->elements[i] = list->elements[i+1];
    }
    
    list->size -= 1;
    list->elements = realloc(list->elements, sizeof(void*) * list->size);
    
    return;
}

void list_remove(const LIST *list, const void *element)
{
    size_t i;
    
    if(list == NULL || element == NULL)
    {
        return;
    }
    
    for(i = 0; i < list->size; ++i)
    {
        if(list->elements[i] == element)
        {
            list_remove_at(list, i);
            
            break;
        }
    }
    
    return;
}

void list_sort(const LIST *list, list_sort_handler_t sort_handler)
{
    qsort(list->elements, list->size, sizeof(void*), sort_handler);
}
