#include <stdlib.h>
#include <string.h>

#include <curl/curl.h>

#include "auth.h"
#include "helper/list.h"
#include "helper/md5.h"

#define API_URL "http://ws.audioscrobbler.com/2.0/"
#define BASE_USERAGENT "lastfm"

CURLM *curl_multi = NULL;

void lastfm_init(void)
{
    curl_global_init(CURL_GLOBAL_ALL);
    curl_multi = curl_multi_init();
}

void lastfm_shutdown(void)
{
    curl_multi_cleanup(curl_multi);
    curl_global_cleanup();
}

size_t method_response(const char *partial_response, const size_t size, const size_t nmemb, char **response)
{
    *response = realloc(*response, (size * nmemb) + sizeof(char));
    strncat(*response, partial_response, nmemb);
    return nmemb;
}

char *create_api_signature(const char *post_in, const LASTFM_AUTH *auth)
{
    int list_sort_handler(const void *parameter1, const void *parameter2)
    {
        return strcmp((const char*)*(const void**)parameter1, (const char*)*(const void**)parameter2);
    }

    LIST *parameters;
    char *post, *post_stripped, *parameter, *api_sig;
    const char *delimiter = "&";
    size_t i, j;
    
    parameters = list_new();
    asprintf(&post, "%s", post_in);
    for(parameter = strtok(post, delimiter); parameter != NULL; parameter = strtok(NULL, delimiter))
    {
        char *element;
    
        if(strcmp(parameter, "format=json") != 0)
        {
            asprintf(&element, "%s", parameter);
            list_add(parameters, element);
        }
    }
    free(post);
    list_sort(parameters, list_sort_handler);
    
    post = malloc(sizeof(char) * (strlen(post_in) + 1));
    post[0] = '\0';
    for(i = 0; i < parameters->size; ++i)
    {
        strcat(post, parameters->elements[i]);
    }
    
    post = realloc(post, sizeof(char) * (strlen(post) + strlen(auth->secret) + 1));
    strcat(post, auth->secret);
    
    post_stripped = malloc(sizeof(char) * (strlen(post) + 1));
    for(i = j = 0; i < strlen(post); ++i)
    {
        if(post[i] != '=')
        {
            post_stripped[j] = post[i];
            ++j;
        }
    }

    api_sig = md5(post_stripped);
    free(post);
    free(post_stripped);
    parameters->free_handler = free;
    list_delete(parameters);
    
    return api_sig;
}

char *lastfm_method(const char *method, const LASTFM_AUTH *auth)
{
    CURL *curl;
    char *post, *post_unsigned, *api_signature, *response;

    /** Prepare data **/
    curl = curl_easy_init();
    asprintf(&post_unsigned, "method=%s&api_key=%s&format=json", method, auth->api_key);
    api_signature = create_api_signature(post_unsigned, auth);
    asprintf(&post, "%s&api_sig=%s", post_unsigned, api_signature);
    
    free(post_unsigned);
    free(api_signature);

    /** Set options **/
    /* Callback options */
    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, method_response);
    curl_easy_setopt(curl, CURLOPT_WRITEDATA, &response);
    /* Network options */
    curl_easy_setopt(curl, CURLOPT_URL, API_URL);
    /* HTTP options */
    curl_easy_setopt(curl, CURLOPT_POST, 1);
    curl_easy_setopt(curl, CURLOPT_COPYPOSTFIELDS, post);
    free(post);
    curl_easy_setopt(curl, CURLOPT_USERAGENT, BASE_USERAGENT);
    /* Protocol options */
    curl_easy_setopt(curl, CURLOPT_TIMEOUT, 10);

    /** Do it **/
    response = malloc(sizeof(char));
    response[0] = '\0';
    curl_easy_perform(curl);
    
    /** Clean up **/
    curl_easy_cleanup(curl);
    
    return response;
}
