#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <jansson.h>

#include "shouts.h"
#include "auth.h"
#include "lastfm.h"

#include "helper/list.h"

LASTFM_SHOUT *shout_new(void)
{
    LASTFM_SHOUT *shout;
    
    shout = malloc(sizeof(LASTFM_SHOUT));
    
    shout->body = NULL;
    shout->author = NULL;
    shout->date = malloc(sizeof(struct tm));
    
    return shout;
}

void shout_delete(LASTFM_SHOUT *shout)
{
    if(shout == NULL)
    {
        return;
    }
    
    if(shout->date != NULL)
    {
        free(shout->body);
        free(shout->author);
        free(shout->date);
    }
    
    free(shout);
    
    return;
}

LASTFM_SHOUTS *shouts_new(void)
{
    LASTFM_SHOUTS *shouts;
    
    shouts = malloc(sizeof(LASTFM_SHOUTS));
    
    shouts->artist = NULL;
    shouts->album = NULL;
    shouts->track = NULL;
    shouts->event = NULL;
    
    shouts->shouts = list_new();
    shouts->shouts->free_handler = (list_free_handler_t)shout_delete;
    
    return shouts;
}

void lastfm_shouts_delete(LASTFM_SHOUTS *shouts)
{
    if(shouts == NULL)
    {
        return;
    }
    
    /*free(shouts->artist);
    free(shouts->album);
    free(shouts->track);
    free(shouts->event);*/
    
    printf("%s\n%s\n%s\n%s\n", shouts->artist, shouts->album, shouts->track, shouts->event);
    
    list_delete(shouts->shouts);
    free(shouts);
    
    return;
}

LASTFM_SHOUTS *getShouts(const char *user,
                         const char *artist,
                         const char *album,
                         const char *track,
                         const char *mbid,
                         const char *event,
                         const unsigned int page,
                         const unsigned int limit,
                         const bool autocorrect,
                         LASTFM_AUTH *auth,
                         const PAGE_TYPE page_type)
{
    char *method, *response;
    char *shouts_artist, *shouts_album, *shouts_track, *shouts_event,
         *shouts_page, *shouts_perPage, *shouts_totalPages, *shouts_total;
    const char *package, *key[3], *value[3];
    LASTFM_SHOUTS *shouts;
    json_t *json_shouts;
    json_error_t json_error;
    unsigned int i;
    
    memset(key, 0, sizeof(key));
    memset(value, 0, sizeof(value));
    
    switch(page_type)
    {
        case PAGE_ALBUM:
        {
            package = "album";
        }
        break;
        case PAGE_ARTIST:
        {
            package = "artist";
        }
        break;
        case PAGE_EVENT:
        {
            package = "event";
        }
        break;
        case PAGE_TRACK:
        {
            package = "track";
        }
        break;
        case PAGE_USER:
        {
            package = "user";
        }
        break;
    }
    
    switch(page_type)
    {
        case PAGE_ALBUM:
        case PAGE_ARTIST:
        case PAGE_TRACK:
        {
            static char autocorrect_str[2];
        
            if(mbid == NULL)
            {
                key[0] = "&artist=";
                value[0] = artist;
                
                switch(page_type)
                {
                    case PAGE_ALBUM:
                    {
                        key[1] = "&album=";
                        value[1] = album;
                    }
                    break;
                    case PAGE_TRACK:
                    {
                        key[1] = "&track=";
                        value[1] = track;
                    }
                    break;
                }
            }
            else
            {
                key[0] = "&mbid=";
                value[0] = mbid;
            }
            
            key[2] = "&autocorrect=";
            snprintf((char*)&autocorrect_str, sizeof(autocorrect_str) - 1, "%d", autocorrect);
            value[2] = (char*)&autocorrect_str;
        }
        break;
        case PAGE_EVENT:
        {
            key[0] = "&event=";
            value[0] = event;
        }
        break;
        case PAGE_USER:
        {
            key[0] = "&user=";
            value[0] = user;
        }
        break;
    }
    
    asprintf(&method, "%s.getShouts%s%s%s%s%s%s&page=%d&limit=%d",
             package, key[0], value[0], key[1], value[1], key[2], value[2],
             page, limit);
    response = lastfm_method(method, auth);
    free(method);
    
    json_shouts = json_loads(response, 0, &json_error);
    free(response);
    
    shouts = shouts_new();
    json_shouts = json_object_get(json_shouts, "shouts");
    json_unpack_ex(json_object_get(json_shouts, "@attr"), &json_error, JSON_STRICT, "{s?s, s?s, s?s, s?s, s:s, s:s, s:s, s:s}",
                   "artist", &shouts_artist,
                   "album", &shouts_album,
                   "track", &shouts_track,
                   "event", &shouts_event,
                   "page", &shouts_page,
                   "perPage", &shouts_perPage,
                   "totalPages", &shouts_totalPages,
                   "total", &shouts_total);
                
    switch(shouts->
    asprintf(&(shouts->artist), "%s", shouts_artist);
    asprintf(&(shouts->album), "%s", shouts_album);
    asprintf(&(shouts->track), "%s", shouts_track);
    asprintf(&(shouts->event), "%s", shouts_event);
    shouts->page = atoi(shouts_page);
    shouts->perPage = atoi(shouts_perPage);
    shouts->totalPages = atoi(shouts_totalPages);
    shouts->total = atoi(shouts_total);
    
    json_shouts = json_object_get(json_shouts, "shout");
    for(i = 0; i < json_array_size(json_shouts); ++i)
    {
        LASTFM_SHOUT *shout;
        json_t *json_shout;
        char *shout_body, *shout_author, *shout_date;

        shout = shout_new();
        
        json_shout = json_array_get(json_shouts, i);
        json_unpack_ex(json_shout, &json_error, JSON_STRICT, "{s:s, s:s, s:s}",
                       "body", &shout_body,
                       "author", &shout_author,
                       "date", &shout_date);
                       
        asprintf(&(shout->body), "%s", shout_body);
        asprintf(&(shout->author), "%s", shout_author);
        strptime(shout_date, "%a, %d %b %Y %T", shout->date);
        
        list_add(shouts->shouts, shout);
    }
    
    json_decref(json_shouts);
    
    return shouts;
}
